#
# fishing.R,  6 Aug 23
#
# Data from:
# Evidence-Based Software Portfolio Management
# Hennie Huijgens

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)


hh=read.csv("EBSPM_Research_Repository_v07072017.csv", as.is=TRUE, sep=";")

hh=subset(hh, Organization != "DutchCo") # Only four rows
hh=subset(hh, Actual_cost_EUR > 1e3) # One 'outlier'

# Flags
# Release_based_working
# Once_only_project
# Multi_application_release
# Phased_project
# Dependencies_with_other_applications
# Single_application
# Migration_project
# Problems_with_external_supplier
# Security_related_project
# Package_of_the_shelf
# Package_with_customization
# Steady_heartbeat
# Fixed_experienced_team
# Many_team_changes_unexperienced_team
# Legacy_application
# Technology_driven_project
# Rules_and_regulations_driven_project
# Business_driven_project
# Pilot_or_proof_of_concept 

# Plot points for one organization, fit regression line and
# find one/two sd cost ratios
company_FP=function(df)
{
points(df$Functional_size_FP, df$Actual_cost_EUR, col=df$col_str[1])
df_mod=glm(log(Actual_cost_EUR) ~ log(Functional_size_FP), data=df)
pred=predict(df_mod, newdata=data.frame(Functional_size_FP=x_bounds), se.fit=TRUE)
lines(x_bounds, exp(pred$fit), col=df$col_str[1])
# print(summary(df_mod))

MSE=sum(df_mod$residuals^2)/(length(df_mod$residuals)-2)
# Variances, but not sd, can be added
pred_se=sqrt(pred$se.fit^2+MSE)

# Cost ratio for estimates falling within 1 and 1.96 sd
print(coef(df_mod))
print(c(df$Organization[1], mean(exp(pred_se)), mean(exp(1.96*pred_se))))
}


# Data quality for Effort reported as low (thesis pp44)
plot(hh$Functional_size_FP, hh$Actual_effort_hours, log="xy",
	xlab="FP", ylab="Actual effort (hours)")


pal_col=rainbow(2)

plot(hh$Functional_size_FP, hh$Actual_cost_EUR, log="xy", col=pal_col[2],
	xlab="FP", ylab="Actual cost (EUR)")
lines(c(1, 1e4), 2.5e3*c(1, 1e4), col="grey")

fpc_mod=glm(log(Actual_cost_EUR) ~ log(Functional_size_FP), data=hh)
pred=predict(fpc_mod, newdata=data.frame(Functional_size_FP=x_bounds), se.fit=TRUE)
lines(x_bounds, exp(pred$fit), col=pal_col[1])

# MSE=sum(fpc_mod$residuals^2)/(length(fpc_mod$residuals)-2)
# # Variances, but not sd, can be added
# pred_se=sqrt(pred$se.fit^2+MSE)
# 
# # One standard deviation, 68.5% confidence interval
# lines(x_bounds, exp(pred$fit+1*pred_se), col=pal_col[4])
# lines(x_bounds, exp(pred$fit-1*pred_se), col=pal_col[4])
# 
# # 95% confidence interval, almost two standard deviations
# lines(x_bounds, exp(pred$fit+1.96*pred_se), col=pal_col[2])
# lines(x_bounds, exp(pred$fit-1.96*pred_se), col=pal_col[2])
# 
# legend(x="bottomright", legend=c("Fitted model", "95% prediction interval", "Tasks", "1 sd (68%) prediction interval"),
# 			bty="n", fill=pal_col, cex=1.2)

dev.copy(dev=png, file="FP-cost-combined.png")
dev.off()


# One and two standard deviation confidence intervals
x_1sd=exp(pred$fit+pred_se)/exp(pred$fit)  # -> exp(pred_se)
x_2sd=exp(pred$fit+1.96*pred_se)/exp(pred$fit)  # -> exp(1.96*pred_se)

mean(x_1sd)
# [1] 2.504996
mean(x_2sd)
# [1] 6.048724

plot(hh$Actual_cost_EUR, hh$Actual_duration_months, log="xy",
	xlab="Actual Cost", ylab="Actual duration (months)")

plot(hh$Actual_duration_months, hh$Actual_cost_EUR, log="xy",
	xlab="Actual_duration (months)", ylab="Actual cost (EUR)")

fpc_mod=glm(log(Actual_cost_EUR) ~ log(Functional_size_FP), data=hh)
summary(fpc_mod)


fpc_mod=glm(log(Actual_cost_EUR) ~ log(Functional_size_FP)+Organization+Development_method+Fixed_experienced_team+Business_domain, data=hh)
summary(fpc_mod)


fpc_mod=glm(log(Actual_cost_EUR) ~ log(Functional_size_FP)+Organization+Development_method+Business_domain
+ Fixed_experienced_team
+ Multi_application_release
+ Phased_project
+ Single_application
+ Migration_project
+ Security_related_project
+ Many_team_changes_unexperienced_team
, data=hh)
summary(fpc_mod)

exp(coef(fpc_mod))

# Separate out data by Organization
u_Org=unique(hh$Organization)
pal_col=rainbow(length(u_Org))
hh$col_str=mapvalues(hh$Organization, u_Org, pal_col)

x_bounds=seq(5, 5e3, by=3)
plot(hh$Functional_size_FP, hh$Actual_cost_EUR, type="n", log="xy",
	xlab="FP", ylab="Actual cost (EUR)")

d_ply(hh, .(Organization), company_FP)

legend("bottomright", legend=u_Org,
                        fill=pal_col, border="white", bty="n")

dev.copy(dev=png, file="Org-FP-cost.png")
dev.off()


